@@ -16,7 +16,7 @@ class AuthViewsTestCase(TestCase):
     Helper base class for all the follow test cases.
     """
     fixtures = ['authtestdata.json']
-    urls = 'django.contrib.auth.urls'
+    urls = 'django.contrib.auth.tests.urls'
 
     def setUp(self):
         self.old_LANGUAGES = settings.LANGUAGES
@@ -82,6 +82,18 @@ class PasswordResetTest(AuthViewsTestCase):
         self.assertEquals(response.status_code, 200)
         self.assert_("The password reset link was invalid" in response.content)
 
+    def test_confirm_invalid_user(self):
+        # Ensure that we get a 200 response for a non-existant user, not a 404
+        response = self.client.get('/reset/123456-1-1/')
+        self.assertEquals(response.status_code, 200)
+        self.assert_("The password reset link was invalid" in response.content)
+
+    def test_confirm_overflow_user(self):
+        # Ensure that we get a 200 response for a base36 user id that overflows int
+        response = self.client.get('/reset/zzzzzzzzzzzzz-1-1/')
+        self.assertEquals(response.status_code, 200)
+        self.assert_("The password reset link was invalid" in response.content)
+
     def test_confirm_invalid_post(self):
         # Same as test_confirm_invalid, but trying
         # to do a POST instead.
@@ -179,9 +191,12 @@ class LoginTest(AuthViewsTestCase):
     def test_current_site_in_context_after_login(self):
         response = self.client.get(reverse('django.contrib.auth.views.login'))
         self.assertEquals(response.status_code, 200)
-        site = Site.objects.get_current()
-        self.assertEquals(response.context['site'], site)
-        self.assertEquals(response.context['site_name'], site.name)
+        if Site._meta.installed:
+            site = Site.objects.get_current()
+            self.assertEquals(response.context['site'], site)
+            self.assertEquals(response.context['site_name'], site.name)
+        else:
+            self.assertTrue(isinstance(response.context['site'], RequestSite))
         self.assert_(isinstance(response.context['form'], AuthenticationForm),
                      'Login form is not an AuthenticationForm')
 
@@ -205,7 +220,7 @@ class LoginTest(AuthViewsTestCase):
                 }
             )
             self.assertEquals(response.status_code, 302)
-            self.failIf(bad_url in response['Location'], "%s should be blocked" % bad_url)
+            self.assertFalse(bad_url in response['Location'], "%s should be blocked" % bad_url)
 
         # Now, these URLs have an other URL as a GET parameter and therefore
         # should be allowed
@@ -222,7 +237,7 @@ class LoginTest(AuthViewsTestCase):
                 }
             )
             self.assertEquals(response.status_code, 302)
-            self.failUnless('/view/?param=%s' % url_ in response['Location'], "/view/?param=%s should be allowed" % url_)
+            self.assertTrue('/view/?param=%s' % url_ in response['Location'], "/view/?param=%s should be allowed" % url_)
 
 
 class LogoutTest(AuthViewsTestCase):
@@ -249,6 +264,25 @@ class LogoutTest(AuthViewsTestCase):
         self.assert_('Logged out' in response.content)
         self.confirm_logged_out()
 
+    def test_14377(self):
+        # Bug 14377
+        self.login()
+        response = self.client.get('/logout/')
+        self.assertTrue('site' in response.context)
+
+    def test_logout_with_overridden_redirect_url(self):
+        # Bug 11223
+        self.login()
+        response = self.client.get('/logout/next_page/')
+        self.assertEqual(response.status_code, 302)
+        self.assert_(response['Location'].endswith('/somewhere/'))
+
+        response = self.client.get('/logout/next_page/?next=/login/')
+        self.assertEqual(response.status_code, 302)
+        self.assert_(response['Location'].endswith('/login/'))
+
+        self.confirm_logged_out()
+
     def test_logout_with_next_page_specified(self):
         "Logout with next_page option given redirects to specified resource"
         self.login()
@@ -272,3 +306,45 @@ class LogoutTest(AuthViewsTestCase):
         self.assertEqual(response.status_code, 302)
         self.assert_(response['Location'].endswith('/somewhere/'))
         self.confirm_logged_out()
+
+    def test_security_check(self, password='password'):
+        logout_url = reverse('django.contrib.auth.views.logout')
+
+        # Those URLs should not pass the security check
+        for bad_url in ('http://example.com',
+                        'https://example.com',
+                        'ftp://exampel.com',
+                        '//example.com'
+                        ):
+            nasty_url = '%(url)s?%(next)s=%(bad_url)s' % {
+                'url': logout_url,
+                'next': REDIRECT_FIELD_NAME,
+                'bad_url': urllib.quote(bad_url)
+            }
+            self.login()
+            response = self.client.get(nasty_url)
+            self.assertEquals(response.status_code, 302)
+            self.assertFalse(bad_url in response['Location'],
+                             "%s should be blocked" % bad_url)
+            self.confirm_logged_out()
+
+        # These URLs *should* still pass the security check
+        for good_url in ('/view/?param=http://example.com',
+                         '/view/?param=https://example.com',
+                         '/view?param=ftp://exampel.com',
+                         'view/?param=//example.com',
+                         'https:///',
+                         '//testserver/',
+                         '/url%20with%20spaces/', # see ticket #12534
+                         ):
+            safe_url = '%(url)s?%(next)s=%(good_url)s' % {
+                'url': logout_url,
+                'next': REDIRECT_FIELD_NAME,
+                'good_url': urllib.quote(good_url)
+            }
+            self.login()
+            response = self.client.get(safe_url)
+            self.assertEquals(response.status_code, 302)
+            self.assertTrue(good_url in response['Location'],
+                            "%s should be allowed" % good_url)
+            self.confirm_logged_out()
