@@ -1,13 +1,15 @@
 import datetime
+import pickle
 from decimal import Decimal
+from operator import attrgetter
 
-from django.core.exceptions import FieldError
 from django.conf import settings
-from django.test import TestCase, Approximate
+from django.core.exceptions import FieldError
 from django.db import DEFAULT_DB_ALIAS
-from django.db.models import Count, Max, Avg, Sum, F
+from django.db.models import Count, Max, Avg, Sum, StdDev, Variance, F, Q
+from django.test import TestCase, Approximate
 
-from regressiontests.aggregation_regress.models import *
+from models import Author, Book, Publisher, Clues, Entries, HardbackBook
 
 
 def run_stddev_tests():
@@ -135,7 +137,6 @@ class AggregationTests(TestCase):
             contact_id=3,
             id=2,
             isbn=u'067232959',
-            manufacture_cost=11.545,
             mean_auth_age=45.0,
             name='Sams Teach Yourself Django in 24 Hours',
             pages=528,
@@ -144,6 +145,8 @@ class AggregationTests(TestCase):
             publisher_id=2,
             rating=3.0
         )
+        # Different DB backends return different types for the extra select computation
+        self.assertTrue(obj.manufacture_cost == 11.545 or obj.manufacture_cost == Decimal('11.545'))
 
         # Order of the annotate/extra in the query doesn't matter
         obj = Book.objects.extra(select={'manufacture_cost' : 'price * .5'}).annotate(mean_auth_age=Avg('authors__age')).get(pk=2)
@@ -151,7 +154,6 @@ class AggregationTests(TestCase):
             contact_id=3,
             id=2,
             isbn=u'067232959',
-            manufacture_cost=11.545,
             mean_auth_age=45.0,
             name=u'Sams Teach Yourself Django in 24 Hours',
             pages=528,
@@ -160,14 +162,18 @@ class AggregationTests(TestCase):
             publisher_id=2,
             rating=3.0
         )
+        # Different DB backends return different types for the extra select computation
+        self.assertTrue(obj.manufacture_cost == 11.545 or obj.manufacture_cost == Decimal('11.545'))
 
         # Values queries can be combined with annotate and extra
         obj = Book.objects.annotate(mean_auth_age=Avg('authors__age')).extra(select={'manufacture_cost' : 'price * .5'}).values().get(pk=2)
+        manufacture_cost = obj['manufacture_cost']
+        self.assertTrue(manufacture_cost == 11.545 or manufacture_cost == Decimal('11.545'))
+        del obj['manufacture_cost']
         self.assertEqual(obj, {
             "contact_id": 3,
             "id": 2,
             "isbn": u"067232959",
-            "manufacture_cost": 11.545,
             "mean_auth_age": 45.0,
             "name": u"Sams Teach Yourself Django in 24 Hours",
             "pages": 528,
@@ -180,11 +186,13 @@ class AggregationTests(TestCase):
         # The order of the (empty) values, annotate and extra clauses doesn't
         # matter
         obj = Book.objects.values().annotate(mean_auth_age=Avg('authors__age')).extra(select={'manufacture_cost' : 'price * .5'}).get(pk=2)
+        manufacture_cost = obj['manufacture_cost']
+        self.assertTrue(manufacture_cost == 11.545 or manufacture_cost == Decimal('11.545'))
+        del obj['manufacture_cost']
         self.assertEqual(obj, {
             'contact_id': 3,
             'id': 2,
             'isbn': u'067232959',
-            'manufacture_cost': 11.545,
             'mean_auth_age': 45.0,
             'name': u'Sams Teach Yourself Django in 24 Hours',
             'pages': 528,
@@ -475,11 +483,49 @@ class AggregationTests(TestCase):
             lambda b: b.name
         )
 
+    def test_duplicate_alias(self):
+        # Regression for #11256 - duplicating a default alias raises ValueError.
+        self.assertRaises(ValueError, Book.objects.all().annotate, Avg('authors__age'), authors__age__avg=Avg('authors__age'))
+
+    def test_field_name_conflict(self):
+        # Regression for #11256 - providing an aggregate name that conflicts with a field name on the model raises ValueError
+        self.assertRaises(ValueError, Author.objects.annotate, age=Avg('friends__age'))
+
+    def test_m2m_name_conflict(self):
+        # Regression for #11256 - providing an aggregate name that conflicts with an m2m name on the model raises ValueError
+        self.assertRaises(ValueError, Author.objects.annotate, friends=Count('friends'))
+
+    def test_values_queryset_non_conflict(self):
+        # Regression for #14707 -- If you're using a values query set, some potential conflicts are avoided.
+
+        # age is a field on Author, so it shouldn't be allowed as an aggregate.
+        # But age isn't included in the ValuesQuerySet, so it is.
+        results = Author.objects.values('name').annotate(age=Count('book_contact_set')).order_by('name')
+        self.assertEquals(len(results), 9)
+        self.assertEquals(results[0]['name'], u'Adrian Holovaty')
+        self.assertEquals(results[0]['age'], 1)
+
+        # Same problem, but aggregating over m2m fields
+        results = Author.objects.values('name').annotate(age=Avg('friends__age')).order_by('name')
+        self.assertEquals(len(results), 9)
+        self.assertEquals(results[0]['name'], u'Adrian Holovaty')
+        self.assertEquals(results[0]['age'], 32.0)
+
+        # Same problem, but colliding with an m2m field
+        results = Author.objects.values('name').annotate(friends=Count('friends')).order_by('name')
+        self.assertEquals(len(results), 9)
+        self.assertEquals(results[0]['name'], u'Adrian Holovaty')
+        self.assertEquals(results[0]['friends'], 2)
+
+    def test_reverse_relation_name_conflict(self):
+        # Regression for #11256 - providing an aggregate name that conflicts with a reverse-related name on the model raises ValueError
+        self.assertRaises(ValueError, Author.objects.annotate, book_contact_set=Avg('friends__age'))
+
     def test_pickle(self):
         # Regression for #10197 -- Queries with aggregates can be pickled.
         # First check that pickling is possible at all. No crash = success
         qs = Book.objects.annotate(num_authors=Count('authors'))
-        out = pickle.dumps(qs)
+        pickle.dumps(qs)
 
         # Then check that the round trip works.
         query = qs.query.get_compiler(qs.db).as_sql()[0]
@@ -543,12 +589,12 @@ class AggregationTests(TestCase):
         )
 
         publishers = Publisher.objects.filter(id__in=[1, 2])
-        self.assertQuerysetEqual(
-            publishers, [
+        self.assertEqual(
+            sorted(p.name for p in publishers),
+            [
                 "Apress",
                 "Sams"
-            ],
-            lambda p: p.name
+            ]
         )
 
         publishers = publishers.annotate(n_books=Count("book"))
@@ -557,12 +603,12 @@ class AggregationTests(TestCase):
             2
         )
 
-        self.assertQuerysetEqual(
-            publishers, [
+        self.assertEqual(
+            sorted(p.name for p in publishers),
+            [
                 "Apress",
-                "Sams",
-            ],
-            lambda p: p.name
+                "Sams"
+            ]
         )
 
         books = Book.objects.filter(publisher__in=publishers)
@@ -574,12 +620,12 @@ class AggregationTests(TestCase):
             ],
             lambda b: b.name
         )
-        self.assertQuerysetEqual(
-            publishers, [
+        self.assertEqual(
+            sorted(p.name for p in publishers),
+            [
                 "Apress",
-                "Sams",
-            ],
-            lambda p: p.name
+                "Sams"
+            ]
         )
 
         # Regression for 10666 - inherited fields work with annotations and
@@ -619,64 +665,180 @@ class AggregationTests(TestCase):
             lambda: Book.objects.annotate(mean_age=Avg('authors__age')).annotate(Avg('mean_age'))
         )
 
+    def test_empty_filter_count(self):
+        self.assertEqual(
+            Author.objects.filter(id__in=[]).annotate(Count("friends")).count(),
+            0
+        )
+
+    def test_empty_filter_aggregate(self):
+        self.assertEqual(
+            Author.objects.filter(id__in=[]).annotate(Count("friends")).aggregate(Count("pk")),
+            {"pk__count": None}
+        )
+
+    def test_annotate_and_join(self):
+        self.assertEqual(
+            Author.objects.annotate(c=Count("friends__name")).exclude(friends__name="Joe").count(),
+            Author.objects.count()
+        )
+
+    def test_f_expression_annotation(self):
+        # Books with less than 200 pages per author.
+        qs = Book.objects.values("name").annotate(
+            n_authors=Count("authors")
+        ).filter(
+            pages__lt=F("n_authors") * 200
+        ).values_list("pk")
+        self.assertQuerysetEqual(
+            Book.objects.filter(pk__in=qs), [
+                "Python Web Development with Django"
+            ],
+            attrgetter("name")
+        )
+
+    def test_values_annotate_values(self):
+        qs = Book.objects.values("name").annotate(
+            n_authors=Count("authors")
+        ).values_list("pk", flat=True)
+        self.assertEqual(list(qs), list(Book.objects.values_list("pk", flat=True)))
+
+    def test_having_group_by(self):
+        # Test that when a field occurs on the LHS of a HAVING clause that it
+        # appears correctly in the GROUP BY clause
+        qs = Book.objects.values_list("name").annotate(
+            n_authors=Count("authors")
+        ).filter(
+            pages__gt=F("n_authors")
+        ).values_list("name", flat=True)
+        # Results should be the same, all Books have more pages than authors
+        self.assertEqual(
+            list(qs), list(Book.objects.values_list("name", flat=True))
+        )
+
+    def test_annotation_disjunction(self):
+        qs = Book.objects.annotate(n_authors=Count("authors")).filter(
+            Q(n_authors=2) | Q(name="Python Web Development with Django")
+        )
+        self.assertQuerysetEqual(
+            qs, [
+                "Artificial Intelligence: A Modern Approach",
+                "Python Web Development with Django",
+                "The Definitive Guide to Django: Web Development Done Right",
+            ],
+            attrgetter("name")
+        )
+
+        qs = Book.objects.annotate(n_authors=Count("authors")).filter(
+            Q(name="The Definitive Guide to Django: Web Development Done Right") | (Q(name="Artificial Intelligence: A Modern Approach") & Q(n_authors=3))
+        )
+        self.assertQuerysetEqual(
+            qs, [
+                "The Definitive Guide to Django: Web Development Done Right",
+            ],
+            attrgetter("name")
+        )
+
+        qs = Publisher.objects.annotate(
+            rating_sum=Sum("book__rating"),
+            book_count=Count("book")
+        ).filter(
+            Q(rating_sum__gt=5.5) | Q(rating_sum__isnull=True)
+        ).order_by('pk')
+        self.assertQuerysetEqual(
+            qs, [
+                "Apress",
+                "Prentice Hall",
+                "Jonno's House of Books",
+            ],
+            attrgetter("name")
+        )
+
+        qs = Publisher.objects.annotate(
+            rating_sum=Sum("book__rating"),
+            book_count=Count("book")
+        ).filter(
+            Q(pk__lt=F("book_count")) | Q(rating_sum=None)
+        ).order_by("pk")
+        self.assertQuerysetEqual(
+            qs, [
+                "Apress",
+                "Jonno's House of Books",
+            ],
+            attrgetter("name")
+        )
+
+    def test_quoting_aggregate_order_by(self):
+        qs = Book.objects.filter(
+            name="Python Web Development with Django"
+        ).annotate(
+            authorCount=Count("authors")
+        ).order_by("authorCount")
+        self.assertQuerysetEqual(
+            qs, [
+                ("Python Web Development with Django", 3),
+            ],
+            lambda b: (b.name, b.authorCount)
+        )
+
     if run_stddev_tests():
         def test_stddev(self):
             self.assertEqual(
                 Book.objects.aggregate(StdDev('pages')),
-                {'pages__stddev': 311.46}
+                {'pages__stddev': Approximate(311.46, 1)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(StdDev('rating')),
-                {'rating__stddev': 0.60}
+                {'rating__stddev': Approximate(0.60, 1)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(StdDev('price')),
-                {'price__stddev': 24.16}
+                {'price__stddev': Approximate(24.16, 2)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(StdDev('pages', sample=True)),
-                {'pages__stddev': 341.19}
+                {'pages__stddev': Approximate(341.19, 2)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(StdDev('rating', sample=True)),
-                {'rating__stddev': 0.66}
+                {'rating__stddev': Approximate(0.66, 2)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(StdDev('price', sample=True)),
-                {'price__stddev': 26.46}
+                {'price__stddev': Approximate(26.46, 1)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(Variance('pages')),
-                {'pages__variance': 97010.80}
+                {'pages__variance': Approximate(97010.80, 1)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(Variance('rating')),
-                {'rating__variance': 0.36}
+                {'rating__variance': Approximate(0.36, 1)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(Variance('price')),
-                {'price__variance': 583.77}
+                {'price__variance': Approximate(583.77, 1)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(Variance('pages', sample=True)),
-                {'pages__variance': 116412.96}
+                {'pages__variance': Approximate(116412.96, 1)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(Variance('rating', sample=True)),
-                {'rating__variance': 0.44}
+                {'rating__variance': Approximate(0.44, 2)}
             )
 
             self.assertEqual(
                 Book.objects.aggregate(Variance('price', sample=True)),
-                {'price__variance': 700.53}
+                {'price__variance': Approximate(700.53, 2)}
             )
