@@ -3,9 +3,11 @@ from django.conf import settings
 from django.contrib.auth.models import User
 from django.contrib.comments import signals
 from django.contrib.comments.models import Comment
-from regressiontests.comment_tests.models import Article
+from regressiontests.comment_tests.models import Article, Book
 from regressiontests.comment_tests.tests import CommentTestCase
 
+post_redirect_re = re.compile(r'^http://testserver/posted/\?c=(?P<pk>\d+$)')
+
 class CommentViewTests(CommentTestCase):
 
     def testPostCommentHTTPMethods(self):
@@ -43,6 +45,22 @@ class CommentViewTests(CommentTestCase):
         response = self.client.post("/post/", data)
         self.assertEqual(response.status_code, 400)
 
+    def testPostInvalidIntegerPK(self):
+        a = Article.objects.get(pk=1)
+        data = self.getValidData(a)
+        data["comment"] = "This is another comment"
+        data["object_pk"] = u'\ufffd'
+        response = self.client.post("/post/", data)
+        self.assertEqual(response.status_code, 400)
+
+    def testPostInvalidDecimalPK(self):
+        b = Book.objects.get(pk='12.34')
+        data = self.getValidData(b)
+        data["comment"] = "This is another comment"
+        data["object_pk"] = 'cookies'
+        response = self.client.post("/post/", data)
+        self.assertEqual(response.status_code, 400)
+
     def testCommentPreview(self):
         a = Article.objects.get(pk=1)
         data = self.getValidData(a)
@@ -142,13 +160,18 @@ class CommentViewTests(CommentTestCase):
 
         # Connect signals and keep track of handled ones
         received_signals = []
-        excepted_signals = [signals.comment_will_be_posted, signals.comment_was_posted]
-        for signal in excepted_signals:
+        expected_signals = [
+            signals.comment_will_be_posted, signals.comment_was_posted
+        ]
+        for signal in expected_signals:
             signal.connect(receive)
 
         # Post a comment and check the signals
         self.testCreateValidComment()
-        self.assertEqual(received_signals, excepted_signals)
+        self.assertEqual(received_signals, expected_signals)
+
+        for signal in expected_signals:
+            signal.disconnect(receive)
 
     def testWillBePostedSignal(self):
         """
@@ -156,12 +179,13 @@ class CommentViewTests(CommentTestCase):
         actually getting saved
         """
         def receive(sender, **kwargs): return False
-        signals.comment_will_be_posted.connect(receive)
+        signals.comment_will_be_posted.connect(receive, dispatch_uid="comment-test")
         a = Article.objects.get(pk=1)
         data = self.getValidData(a)
         response = self.client.post("/post/", data)
         self.assertEqual(response.status_code, 400)
         self.assertEqual(Comment.objects.count(), 0)
+        signals.comment_will_be_posted.disconnect(dispatch_uid="comment-test")
 
     def testWillBePostedSignalModifyComment(self):
         """
@@ -175,27 +199,35 @@ class CommentViewTests(CommentTestCase):
         signals.comment_will_be_posted.connect(receive)
         self.testCreateValidComment()
         c = Comment.objects.all()[0]
-        self.failIf(c.is_public)
+        self.assertFalse(c.is_public)
 
     def testCommentNext(self):
         """Test the different "next" actions the comment view can take"""
         a = Article.objects.get(pk=1)
         data = self.getValidData(a)
         response = self.client.post("/post/", data)
-        self.assertEqual(response["Location"], "http://testserver/posted/?c=1")
+        location = response["Location"]
+        match = post_redirect_re.match(location)
+        self.assertTrue(match != None, "Unexpected redirect location: %s" % location)
 
         data["next"] = "/somewhere/else/"
         data["comment"] = "This is another comment"
         response = self.client.post("/post/", data)
-        self.assertEqual(response["Location"], "http://testserver/somewhere/else/?c=2")
+        location = response["Location"]
+        match = re.search(r"^http://testserver/somewhere/else/\?c=\d+$", location)
+        self.assertTrue(match != None, "Unexpected redirect location: %s" % location)
 
     def testCommentDoneView(self):
         a = Article.objects.get(pk=1)
         data = self.getValidData(a)
         response = self.client.post("/post/", data)
-        response = self.client.get("/posted/", {'c':1})
+        location = response["Location"]
+        match = post_redirect_re.match(location)
+        self.assertTrue(match != None, "Unexpected redirect location: %s" % location)
+        pk = int(match.group('pk'))
+        response = self.client.get(location)
         self.assertTemplateUsed(response, "comments/posted.html")
-        self.assertEqual(response.context[0]["comment"], Comment.objects.get(pk=1))
+        self.assertEqual(response.context[0]["comment"], Comment.objects.get(pk=pk))
 
     def testCommentNextWithQueryString(self):
         """
@@ -206,7 +238,45 @@ class CommentViewTests(CommentTestCase):
         data["next"] = "/somewhere/else/?foo=bar"
         data["comment"] = "This is another comment"
         response = self.client.post("/post/", data)
-        location = response["Location"]        
+        location = response["Location"]
         match = re.search(r"^http://testserver/somewhere/else/\?foo=bar&c=\d+$", location)
+        self.assertTrue(match != None, "Unexpected redirect location: %s" % location)
+
+    def testCommentPostRedirectWithInvalidIntegerPK(self):
+        """
+        Tests that attempting to retrieve the location specified in the
+        post redirect, after adding some invalid data to the expected
+        querystring it ends with, doesn't cause a server error.
+        """
+        a = Article.objects.get(pk=1)
+        data = self.getValidData(a)
+        data["comment"] = "This is another comment"
+        response = self.client.post("/post/", data)
+        location = response["Location"]
+        broken_location = location + u"\ufffd"
+        response = self.client.get(broken_location)
+        self.assertEqual(response.status_code, 200)
+
+    def testCommentNextWithQueryStringAndAnchor(self):
+        """
+        The `next` key needs to handle already having an anchor. Refs #13411.
+        """
+        # With a query string also.
+        a = Article.objects.get(pk=1)
+        data = self.getValidData(a)
+        data["next"] = "/somewhere/else/?foo=bar#baz"
+        data["comment"] = "This is another comment"
+        response = self.client.post("/post/", data)
+        location = response["Location"]
+        match = re.search(r"^http://testserver/somewhere/else/\?foo=bar&c=\d+#baz$", location)
+        self.failUnless(match != None, "Unexpected redirect location: %s" % location)
+
+        # Without a query string
+        a = Article.objects.get(pk=1)
+        data = self.getValidData(a)
+        data["next"] = "/somewhere/else/#baz"
+        data["comment"] = "This is another comment"
+        response = self.client.post("/post/", data)
+        location = response["Location"]
+        match = re.search(r"^http://testserver/somewhere/else/\?c=\d+#baz$", location)
         self.failUnless(match != None, "Unexpected redirect location: %s" % location)
-        
\ No newline at end of file
